import pytest
import os
import logging
import logging.config
import socket
from selenium import webdriver

from tlib.base.TestHelper import webdriver_chrome_executable, webdriver_ie_executable

def log(logger, level, message):
    """
    @type logger: Logger

    Logger function that validates if a logger object exists, and if not, don't do anything
    """

    if logger is not None:
        levels = { "debug": logger.debug,
                  "info": logger.info,
                  "warn": logger.info,
                  "error": logger.error,
                  "fatal": logger.fatal }
        levels[level](message)

def pytest_addoption(parser):
    group = parser.getgroup("selenium", "Testing with the selenium framework")
    group.addoption(
        "--base_url",
        action="store",
        dest="base_url",
        help="base url of the server used for testing")

    group.addoption(
        "--browser",
        action="store",
        dest="browser",
        help="Browser used for running test cases.")

    group.addoption(
        "--proxy",
        action="store",
        dest="proxy",
        help="""Use provided IP and port as proxy server
            Sample usage:
            py.test --proxy=127.0.0.1:8080""")

def pytest_generate_tests(metafunc):
    """
    Parses command line --browser and generates tlib_tests for the browsers specified
    """
    browsers = metafunc.config.getoption("browser")
    if browsers is not None:
        metafunc.parametrize("setup_browser", browsers.split(","), indirect=True, scope="class")

#noinspection PyShadowingNames,PyUnresolvedReferences
@pytest.fixture(scope='class', autouse=True)
def setup_browser(request, logger):
    """
    @type logger: Logger

    Creates a browser as specified by the command line parameter --browser and --proxy
    """
    if request.cls is None:
        #Not called from a class
        return None
    else:
        global b

        #Don't create a browser object is --browser was not specified
        if not hasattr(request, 'param') or request.param is None:
            log(logger, "info", "Parameter --browser was not specified")
            b = None
        else:
            if request.param == "ie":
                log(logger, "debug", "Creating webdriver object for IE")
                b = webdriver.Ie(webdriver_ie_executable())
            elif request.param == "chrome":
                log(logger, "debug", "Creating webdriver object for Chrome")
                b = webdriver.Chrome(webdriver_chrome_executable())
            elif request.param == "firefox":
                log(logger, "debug", "Creating webdriver object for Firefox")
                b = webdriver.Firefox()
            else:
                #noinspection PyUnresolvedReferences
                log(logger, "warn", "{browser}s is not a valid browser".format(browser=request.param))
                pytest.skip("{browser}s is not a valid browser".format(browser=request.param))


            b.maximize_window()
            request.addfinalizer(b.quit)

        request.cls.browser = b
        return b

#noinspection PyUnresolvedReferences
@pytest.fixture(scope="session")
def base_url(request, logger):
    """
    @type logger: Logger

    Returns value of --base_url command line parameter
    """

    if request.config.getoption("base_url") is None:
        #noinspection PyUnresolvedReferences
        log(logger, "warn", "parameter --base_url was not specified")
        pytest.skip("parameter --base_url was not specified")
    else:
        return request.config.getoption("base_url")

#noinspection PyUnresolvedReferences
@pytest.fixture(scope="session")
def proxy(request, logger):
    """
    @type logger: Logger

    Returns value of --proxy command line parameter
    """

    if request.config.getoption("proxy") is None:
        log(logger, "info", "Parameter --proxy was not specified")
        return None
    else:
        p = request.config.getoption("proxy")
        global port
        global ip
        #noinspection PyBroadException
        try:
            (ip, port) = p.split(":")
        except:
            log(logger, "fatal", "Invalid value for --proxy parameter: {proxy}. Should be an IP and port like this 172.27.108.99:8080")
            #noinspection PyUnresolvedReferences
            pytest.exit("""Invalid value for --proxy parameter: {proxy}
                Should be an IP and port like this 172.27.108.99:8080""")

        #validate ip is valid
        try:
            socket.inet_aton(ip)
        except socket.error:
            log(logger, "fatal", "Proxy server has an invalid IP address: {ip}".format(ip=ip))
            #noinspection PyUnresolvedReferences
            pytest.exit("Proxy server has an invalid IP address: {ip}".format(ip=ip))

        #Validate port is an integer between 1 and 65535
        #noinspection PyBroadException
        try:
            port = int(port, base=10)
        except:
            log(logger, "fatal", "Proxy server has an invalid port: (port)".format(port=port))
            #noinspection PyUnresolvedReferences
            pytest.exit("Proxy server has an invalid port: (port)".format(port=port))

        if int(port) not in range(1, 65535, 1):
            log(logger, "fatal", "Proxy server has an invalid port: (port)".format(port=port))
            #noinspection PyUnresolvedReferences
            pytest.exit("Proxy server has an invalid port: (port)".format(port=port))

        return p

def log(logger, level, message):
    """
    @type logger: Logger

    Logger function that validates if a logger object exists, and if not, don't do anything
    """

    if logger is not None:
        levels = { "debug": logger.debug,
                  "info": logger.info,
                  "warn": logger.info,
                  "error": logger.error,
                  "fatal": logger.fatal }
        levels[level](message)

def pytest_addoption(parser):
    group = parser.getgroup("selenium", "Testing with the selenium framework")
    group.addoption(
        "--base_url",
        action="store",
        dest="base_url",
        help="base url of the server used for testing")

    group.addoption(
        "--browser",
        action="store",
        dest="browser",
        help="Browser used for running test cases.")

    group.addoption(
        "--proxy",
        action="store",
        dest="proxy",
        help="""Use provided IP and port as proxy server
            Sample usage:
            py.test --proxy=127.0.0.1:8080""")

def pytest_generate_tests(metafunc):
    """
    Parses command line --browser and generates tlib_tests for the browsers specified
    """
    browsers = metafunc.config.getoption("browser")
    if browsers is not None:
        metafunc.parametrize("setup_browser", browsers.split(","), indirect=True, scope="class")

#noinspection PyShadowingNames,PyUnresolvedReferences
@pytest.fixture(scope='class', autouse=True)
def setup_browser(request, logger):
    """
    @type logger: Logger

    Creates a browser as specified by the command line parameter --browser and --proxy
    """
    if request.cls is None:
        #Not called from a class
        return None
    else:
        global b

        #Don't create a browser object is --browser was not specified
        if not hasattr(request, 'param') or request.param is None:
            log(logger, "info", "Parameter --browser was not specified")
            b = None
        else:
            if request.param == "ie":
                log(logger, "debug", "Creating webdriver object for IE")
                b = webdriver.Ie(webdriver_ie_executable())
            elif request.param == "chrome":
                log(logger, "debug", "Creating webdriver object for Chrome")
                b = webdriver.Chrome(webdriver_chrome_executable())
            elif request.param == "firefox":
                log(logger, "debug", "Creating webdriver object for Firefox")
                b = webdriver.Firefox()
            else:
                #noinspection PyUnresolvedReferences
                log(logger, "warn", "{browser}s is not a valid browser".format(browser=request.param))
                pytest.skip("{browser}s is not a valid browser".format(browser=request.param))


            b.maximize_window()
            request.addfinalizer(b.quit)

        request.cls.browser = b
        return b

#noinspection PyUnresolvedReferences
@pytest.fixture(scope="session")
def base_url(request, logger):
    """
    @type logger: Logger

    Returns value of --base_url command line parameter
    """

    if request.config.getoption("base_url") is None:
        #noinspection PyUnresolvedReferences
        log(logger, "warn", "parameter --base_url was not specified")
        pytest.skip("parameter --base_url was not specified")
    else:
        return request.config.getoption("base_url")

#noinspection PyUnresolvedReferences
@pytest.fixture(scope="session")
def proxy(request, logger):
    """
    @type logger: Logger

    Returns value of --proxy command line parameter
    """

    if request.config.getoption("proxy") is None:
        log(logger, "info", "Parameter --proxy was not specified")
        return None
    else:
        p = request.config.getoption("proxy")
        global port
        global ip
        #noinspection PyBroadException
        try:
            (ip, port) = p.split(":")
        except:
            log(logger, "fatal", "Invalid value for --proxy parameter: {proxy}. Should be an IP and port like this 172.27.108.99:8080")
            #noinspection PyUnresolvedReferences
            pytest.exit("""Invalid value for --proxy parameter: {proxy}
                Should be an IP and port like this 172.27.108.99:8080""")

        #validate ip is valid
        try:
            socket.inet_aton(ip)
        except socket.error:
            log(logger, "fatal", "Proxy server has an invalid IP address: {ip}".format(ip=ip))
            #noinspection PyUnresolvedReferences
            pytest.exit("Proxy server has an invalid IP address: {ip}".format(ip=ip))

        #Validate port is an integer between 1 and 65535
        #noinspection PyBroadException
        try:
            port = int(port, base=10)
        except:
            log(logger, "fatal", "Proxy server has an invalid port: (port)".format(port=port))
            #noinspection PyUnresolvedReferences
            pytest.exit("Proxy server has an invalid port: (port)".format(port=port))

        if int(port) not in range(1, 65535, 1):
            log(logger, "fatal", "Proxy server has an invalid port: (port)".format(port=port))
            #noinspection PyUnresolvedReferences
            pytest.exit("Proxy server has an invalid port: (port)".format(port=port))

        return p

@pytest.fixture(scope='class', autouse=True)
def setup_logger(request):
    """
    Class to create logger when tlib_tests are started
    """
    if request.cls is None:
        #Not called from a class
        return None
    else:
        # Make sure logger exists before using it
        if "logger" in dir(request.cls):
            if request.cls.logger is None:
                project_folder = request.cls.config_folder()
                logging.config.fileConfig(os.path.join(project_folder, "logging.conf"))
                request.cls.logger = logging.getLogger("root")

                #Flush all handlers at the end
                for h in request.cls.logger.parent.handlers:
                    request.addfinalizer(h.flush)

            return request.cls.logger
        else:
            return None

@pytest.fixture(scope='class')
def logger(request):
    return setup_logger(request)
