from suds.xsd.doctor import Import, ImportDoctor
import suds
from SoapRequestor import SoapRequestor


class SpiraModerator(object):
    """
    Helper class to connect and manipulate the data in SpiraTest
    """

    client = None                        #: Soap client to connect to Spira Soap API
    username = "ypgAutoTester"            #: Automation User username (restricted access to SpiraTest)
    password = "@ut0t3$t3r"                #: Automation User password
    tests = None                        #: Holds retrieved tests from Automation test set in Spiratest
    logger = None                #: logger to send loggin information to.  Logger comes from pytest test definitions
    my_soap = None

    def __init__(self, logger):
        """
        Constructor for class

        Args :
        logger : (logger) instance of a logging object configured in testing project

        """

        self.logger = logger
        wsdl_file = "http://mtljiraqcprod01.ad.ypg.com/SpiraTeam/Services/v4_0/ImportExport.svc?wsdl"
        schema_url = 'http://schemas.microsoft.com/2003/10/Serialization/Arrays'
        schema_import = Import(schema_url)
        schema_doctor = ImportDoctor(schema_import)

        self.my_soap = SoapRequestor(self.logger, wsdl_file, schema_doctor)

    def test_connect(self):
        """
        Authenticates client with SpiraTest.

        Returns : (bool) if connection was successful
        """

        result = False
        try:
            result = self.my_soap.client.service.Connection_Authenticate(self.username, self.password)
        except suds.WebFault, e:
            self.logger.error(str(e))

        self.logger.info("The connection to Spira returned :" + str(result))

        return result

    def update_test_case(self, project_id, test_case_id, test_start, test_end, test_status_id, runner_msg,
                         stack_trace, test_set_id):
        """
        Updates a specified test cases with a specified projects Automation test set with the results of the test execution

        Args :
        project_id : (int) Unique project id from within SpiraTest
        test_case_id : (int) Unique test case id from within SpiraTest
        test_start : (datetime) Time at which the test case started its execution. Doesn't seem to affect Spiratest.
        test_end : (datetime) Time at which the test case ended its execution. Doesn't seem to affect Spiratest.
        runner_msg : (str) Parameters that were used in test execution
        stack_trace : (str) Error message returned from failed test cases

        """

        self.my_soap.client.service.TestRun_RecordAutomated2(self.username, self.password, project_id, None,
                                                             test_case_id, None, test_set_id, None, None, test_start,
                                                             test_end, test_status_id, "None", None, 1,
                                                             runner_msg, stack_trace, 1)

    def get_project_id(self, proj_name):
        """
        Gets the Unique prject id from spiratest for a given project name.  Name comes from project's config file

        Args : proj_name : (str) Name of project under test

        Returns : (int) project id
        """

        self.my_soap.client.service.Connection_Authenticate(self.username, self.password)
        project_list = self.my_soap.client.service.Project_Retrieve()

        project = (proj for proj in project_list[0] if proj["Name"] == proj_name).next()
        self.logger.info("Project Id returned is " + str(project.ProjectId))

        return project.ProjectId

    def get_tests_from_set(self, test_set_id, proj_id):
        """
        Fetches a list of test cases for a given test set id in SpiraTest and stores them in class var "tests"

        Args :
        test_set_id : (int) Automation test set id from Spiratest for project under test
        proj_id : (int) project id from Spiratest for project under test

        """

        self.my_soap.client.service.Connection_Authenticate(self.username, self.password)
        self.my_soap.client.service.Connection_ConnectToProject(proj_id)
        test_set = self.my_soap.client.service.TestCase_RetrieveByTestSetId(test_set_id)
        self.tests = test_set[0]

    def get_test_case_id(self, test_name):
        """
        Gets the unique test case id from SpiraTest for the test case name supplied

        Args : testName : (str) Test case name found in SpiraTest and used in test case decorated to label
                            automated test_support

        Returns : (int) test case id
        """

        my_test = (test for test in self.tests if test["Name"] == test_name).next()

        return my_test.TestCaseId